use crate::filter_expr::{compiler::Compiler, opcode::OpCode};

/// Remove useless jumps
///
/// Useless jumps are any kind of jump that is just going to the next line.
pub fn remove_useless_jumps(mut opcodes: Vec<OpCode>) -> (Vec<OpCode>, bool) {
    // `pos` is the position of the opcode in the new list of opcodes, where the LABELS are
    // ignored and not counted
    let mut pos: usize = 0;
    let label_to_addr = Compiler::labels_to_addr(&opcodes);
    let old_total_opcodes = opcodes.len();
    let new_opcodes = opcodes
        .into_iter()
        .map(|opcode| match &opcode {
            OpCode::LABEL(_) => (pos, opcode),
            _ => {
                pos += 1;
                (pos - 1, opcode)
            }
        })
        .filter_map(|(pos, opcode)| match &opcode {
            OpCode::JEQ(_, addr) | OpCode::JMP(addr) | OpCode::JNE(_, addr) => {
                if label_to_addr.get(addr).map(|x| *x) == pos.checked_add(1).map(|r| r.into()) {
                    None
                } else {
                    Some(opcode)
                }
            }
            _ => Some(opcode),
        })
        .collect::<Vec<_>>();
    let new_total_opcodes = new_opcodes.len();

    (new_opcodes, old_total_opcodes != new_total_opcodes)
}
